@@ -50,7 +50,6 @@ G_DEFINE_DYNAMIC_TYPE (CcBluetoothPanel, cc_bluetooth_panel, CC_TYPE_PANEL)
 struct CcBluetoothPanelPrivate {
 	GtkBuilder          *builder;
 	GtkWidget           *chooser;
-	char                *selected_bdaddr;
 	BluetoothClient     *client;
 	BluetoothKillswitch *killswitch;
 	gboolean             debug;
@@ -69,13 +68,22 @@ launch_command (const char *command)
 	}
 }
 
+static const char *
+cc_bluetooth_panel_get_help_uri (CcPanel *panel)
+{
+  return "help:gnome-help/bluetooth";
+}
+
 static void
 cc_bluetooth_panel_class_init (CcBluetoothPanelClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
 
 	object_class->finalize = cc_bluetooth_panel_finalize;
 
+	panel_class->get_help_uri = cc_bluetooth_panel_get_help_uri;
+
 	g_type_class_add_private (klass, sizeof (CcBluetoothPanelPrivate));
 }
 
@@ -104,10 +112,6 @@ cc_bluetooth_panel_finalize (GObject *object)
 		g_object_unref (self->priv->client);
 		self->priv->client = NULL;
 	}
-	if (self->priv->selected_bdaddr) {
-		g_free (self->priv->selected_bdaddr);
-		self->priv->selected_bdaddr = NULL;
-	}
 
 	G_OBJECT_CLASS (cc_bluetooth_panel_parent_class)->finalize (object);
 }
@@ -256,12 +260,10 @@ cc_bluetooth_panel_update_properties (CcBluetoothPanel *self)
 	gtk_widget_hide (WID ("browse_box"));
 	gtk_widget_hide (WID ("send_box"));
 
-	bdaddr = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (self->priv->chooser));
-
 	/* Remove the extra setup widgets */
-	if (g_strcmp0 (self->priv->selected_bdaddr, bdaddr) != 0)
-		remove_extra_setup_widgets (self);
+	remove_extra_setup_widgets (self);
 
+	bdaddr = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (self->priv->chooser));
 	if (bdaddr == NULL) {
 		gtk_widget_set_sensitive (WID ("properties_vbox"), FALSE);
 		gtk_switch_set_active (button, FALSE);
@@ -332,18 +334,15 @@ cc_bluetooth_panel_update_properties (CcBluetoothPanel *self)
 		}
 
 		/* Extra widgets */
-		if (g_strcmp0 (self->priv->selected_bdaddr, bdaddr) != 0)
-			add_extra_setup_widgets (self, bdaddr);
+		add_extra_setup_widgets (self, bdaddr);
 
 		gtk_label_set_text (GTK_LABEL (WID ("address_label")), bdaddr);
+		g_free (bdaddr);
 
 		gtk_widget_set_sensitive (WID ("button_delete"), TRUE);
 		set_notebook_page (self, NOTEBOOK_PAGE_PROPS);
 	}
 
-	g_free (self->priv->selected_bdaddr);
-	self->priv->selected_bdaddr = bdaddr;
-
 	g_signal_handlers_unblock_by_func (button, switch_connected_active_changed, self);
 }
 
