@@ -179,13 +179,22 @@ cc_display_panel_finalize (GObject *object)
   G_OBJECT_CLASS (cc_display_panel_parent_class)->finalize (object);
 }
 
+static const char *
+cc_display_panel_get_help_uri (CcPanel *panel)
+{
+  return "help:gnome-help/prefs-display";
+}
+
 static void
 cc_display_panel_class_init (CcDisplayPanelClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (CcDisplayPanelPrivate));
 
+  panel_class->get_help_uri = cc_display_panel_get_help_uri;
+
   object_class->constructor = cc_display_panel_constructor;
   object_class->get_property = cc_display_panel_get_property;
   object_class->set_property = cc_display_panel_set_property;
@@ -617,7 +626,7 @@ static void
 rebuild_current_monitor_label (CcDisplayPanel *self)
 {
   char *str, *tmp;
-  GdkColor color;
+  GdkRGBA color;
   gboolean use_color;
 
   if (self->priv->current_output)
@@ -628,7 +637,7 @@ rebuild_current_monitor_label (CcDisplayPanel *self)
         tmp = g_strdup (gnome_rr_output_info_get_display_name (self->priv->current_output));
 
       str = g_strdup_printf ("<b>%s</b>", tmp);
-      gnome_rr_labeler_get_color_for_output (self->priv->labeler, self->priv->current_output, &color);
+      gnome_rr_labeler_get_rgba_for_output (self->priv->labeler, self->priv->current_output, &color);
       use_color = TRUE;
       g_free (tmp);
     }
@@ -644,15 +653,10 @@ rebuild_current_monitor_label (CcDisplayPanel *self)
   if (use_color)
     {
       GdkRGBA black = { 0, 0, 0, 1.0 };
-      GdkRGBA light;
 
-      light.red = color.red / 65535.0;
-      light.green = color.green / 65535.0;
-      light.blue = color.blue / 65535.0;
-      light.alpha = 1.0;
       gtk_widget_override_background_color (self->priv->current_monitor_event_box,
                                             gtk_widget_get_state_flags (self->priv->current_monitor_event_box),
-                                            &light);
+                                            &color);
 
       /* Make the label explicitly black.  We don't want it to follow the
        * theme's colors, since the label is always shown against a light
@@ -804,10 +808,8 @@ rebuild_resolution_combo (CcDisplayPanel *self)
   for (i = 0; modes[i] != NULL; ++i)
     add_mode (self, modes[i], output_width, output_height, preferred_id);
 
-  /* And force the preferred mode in the drop-down (when not in clone mode)
-   * https://bugzilla.gnome.org/show_bug.cgi?id=680969 */
-  if (!gnome_rr_config_get_clone (self->priv->current_configuration))
-   add_mode (self, mode, output_width, output_height, preferred_id);
+  /* And force the preferred mode in the drop-down */
+  add_mode (self, mode, output_width, output_height, preferred_id);
 
   current = make_resolution_string (output_width, output_height);
 
@@ -1960,7 +1962,7 @@ paint_output (CcDisplayPanel *self, cairo_t *cr, int i)
   PangoLayout *layout = get_display_name (self, output);
   PangoRectangle ink_extent, log_extent;
   GdkRectangle viewport;
-  GdkColor output_color;
+  GdkRGBA output_color;
   double r, g, b;
   double available_w;
   double factor;
@@ -2025,10 +2027,10 @@ paint_output (CcDisplayPanel *self, cairo_t *cr, int i)
   cairo_rectangle (cr, x, y, w * scale + 0.5, h * scale + 0.5);
   cairo_clip_preserve (cr);
 
-  gnome_rr_labeler_get_color_for_output (self->priv->labeler, output, &output_color);
-  r = output_color.red / 65535.0;
-  g = output_color.green / 65535.0;
-  b = output_color.blue / 65535.0;
+  gnome_rr_labeler_get_rgba_for_output (self->priv->labeler, output, &output_color);
+  r = output_color.red;
+  g = output_color.green;
+  b = output_color.blue;
 
   if (!gnome_rr_output_info_is_active (output))
     {
