@@ -68,6 +68,7 @@ struct _CcPanelClass
   GtkBinClass parent_class;
 
   GPermission * (* get_permission) (CcPanel *panel);
+  const char  * (* get_help_uri)   (CcPanel *panel);
 };
 
 GType        cc_panel_get_type         (void);
@@ -76,6 +77,8 @@ CcShell*     cc_panel_get_shell        (CcPanel     *panel);
 
 GPermission *cc_panel_get_permission   (CcPanel     *panel);
 
+const char  *cc_panel_get_help_uri     (CcPanel     *panel);
+
 G_END_DECLS
 
 #endif /* __CC_PANEL_H */
